
#include <math.h>

#include <helianthus.h>

#define PLAYER_SPEED  200
#define ENEMY_SPEED   150
#define ARROW_SPEED   200
#define ARROW_DELAY   0.5


Sprite Player;
Animation Playeron;

Sprite Enemy;
Animation Enemyon;

Animation Arrowon;
double playerArrowTime;
double enemyArrowTime;

Group playerArrows;
Group enemyArrows;
int playerHits = 10;
int enemyHits = 10;
int wins = 0;

Animation gameOveron;
Animation gameOverbg;
int gameOver;

Sound gameSound;
Sound gameOverSound;

void init() {
  Playeron = createAnimation("data/Player.png");
  Player = createSpriteEx(512/2, 512-12.5, 25, 25, Playeron);

  Enemyon = createAnimation("data/Enemy.png");
  Enemy = createSpriteEx(512/2, 12.5, 25, 25, Enemyon);

  Arrowon = createAnimation("data/arrow.png");
  playerArrows = createGroup();
  enemyArrows = createGroup();

  gameOveron = createAnimation("data/game-over.png");
  gameOverbg = createAnimation("data/game-over-bg.png");

  gameSound = createSound("data/Radioactive - Imagine Dragons.ogg");
  gameOverSound = createSound("data/Mission to Mars.ogg");
  soundPlay(gameSound, TRUE);

  background(COLOR_BLACK);
}

void draw() {
  saveState();
  double dt = windowGetFrameTime();

  double px = spriteGetX(Player);
  if (keyDown("left")  && playerHits > 0) px -= PLAYER_SPEED*dt;
  if (keyDown("right") && playerHits > 0) px += PLAYER_SPEED*dt;
  if (px < 12.5) px = 12.5;
  if (px > 512-12.5) px = 512-12.5;
  spriteSetX(Player, px);

  playerArrowTime -= dt;
  if (keyDown("space") && playerArrowTime <= 0 && playerHits > 0) {
    Sprite s = createSpriteEx(px, 512-12.5, 25, 25, Arrowon);
    spriteSetVelocityY(s, -ARROW_SPEED);
    groupAdd(playerArrows, s);
    spriteDestroyTimer(s, 5);
    playerArrowTime = ARROW_DELAY;
  }

  double ex = spriteGetX(Enemy);
  if (ex < px && !gameOver) {
    ex += ENEMY_SPEED*dt;
    if (ex > px) ex = px;
  } else {
    ex -= ENEMY_SPEED*dt;
    if (ex < px) ex = px;
  }
  spriteSetX(Enemy, ex);

  enemyArrowTime -= dt;
  if (ex == px && enemyArrowTime <= 0 && !gameOver) {
    Sprite s = createSpriteEx(px, 12.5, 25, 25, Arrowon);
    spriteSetVelocityY(s, ARROW_SPEED);
    spriteDestroyTimer(s, 5);
    groupAdd(enemyArrows, s);
    enemyArrowTime = ARROW_DELAY;
  }

  for(int i = groupGetCount(enemyArrows) - 1; i >= 0; --i) {
    Sprite s = groupGet(enemyArrows, i);
    if (spriteOverlap(s, Player)) {
      playerHits -= 1;
      spriteDestroy(s);
      if (playerHits <= 0) {
        playerHits = 0;
      }
    }
  }

  for(int i = groupGetCount(playerArrows) - 1; i >= 0; --i) {
    Sprite s = groupGet(playerArrows, i);
    if (spriteOverlap(s, Enemy)) {
      enemyHits -= 1;
      spriteDestroy(s);
      if (enemyHits <= 0) {
        spriteSetX(Enemy, 512/2);
        enemyHits = 10;
        playerHits += 5;
        if (playerHits > 10) playerHits = 10;
        ++wins;
      }
    }
  }

  if (playerHits <= 0 && groupGetCount(playerArrows) == 0) {
    soundStop(gameSound);
    soundPlay(gameOverSound, TRUE);
    gameOver = TRUE;
  }

  drawSprites();

  noFill();
  stroke(COLOR_WHITE);
  textf(5, 5, "%d", enemyHits);
  textAlign(HALIGN_LEFT, VALIGN_BOTTOM);
  textf(5, 512 - 5, "%d", playerHits);
  textAlign(HALIGN_RIGHT, VALIGN_BOTTOM);
  textf(512 - 5, 512 - 5, "%d", wins);

  if (gameOver) {
    noStroke();
    fill(COLOR_BLACK);
    rect(0, 0, 512, 512);
    fill(colorByRGBA( 1, 1, 1, (sin(windowGetSeconds()) + 1)/2 ));
    rectTextured(gameOverbg, 0, 0, 512, 512);
    rectTextured(gameOveron, 0, 0, 512, 512);
    if (keyDown("any return")) {
      spriteSetX(Player, 512/2);
      spriteSetX(Enemy, 512/2);
      groupDestroyEach(playerArrows);
      groupDestroyEach(enemyArrows);
      playerHits = 10;
      enemyHits = 10;
      wins = 0;
      gameOver = FALSE;
      soundStop(gameOverSound);
      soundPlay(gameSound, TRUE);
    }
  }

  restoreState();
}

int main() {
  windowSetVariableFrameRate();
  windowSetInit(&init);
  windowSetDraw(&draw);
  windowRun();
  return 0;
}
